#include "device.h"
#include "osd.h"
#include "thread.h"
#include "common.h"
#include <vdr/config.h>
#include <fmp.h>

cDeviceEm84xx::cDeviceEm84xx(void):
		mThread(new cThreadEm84xx()),
		mIsOpen(false)
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	Open();
}

cDeviceEm84xx::~cDeviceEm84xx() 
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	delete mThread;
	Close();
}

void cDeviceEm84xx::Open(void)
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	if (!mIsOpen) {
		FMPOpen(FMPF_PROGRAM, KILOBYTE(32), 10, NULL, 0);
		FMPSet(FMPI_VIDEOOUT, FMPV_VIDEOOUT_COMPOSITE | FMPV_VIDEOOUT_TV | 
							  FMPV_VIDEOOUT_PAL);
		SetVideoFormat(Setup.VideoFormat);
		FMPSetVideoPortDimensions(720, 576);
		mIsOpen = true;
	}
}

void cDeviceEm84xx::Close(void)
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	if (mIsOpen) {
		FMPClose();
		mIsOpen = false;
	}
}
	
bool cDeviceEm84xx::Initialize(void) 
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);

#ifdef EM_HASNO_DVD
	DWORD res = MPEGDriverEntry(NO_DRIVE);
#else
	#ifdef EM_DVD_PRIMARY
		DWORD res = MPEGDriverEntry(PRIMARY_MASTER);
	#else
		DWORD res = MPEGDriverEntry(SECONDARY_MASTER);
	#endif
#endif

	if (res == 0) {
		new cDeviceEm84xx;
		return true;
	}
	return false;
}

void cDeviceEm84xx::Destruct(void)
{
	MPEGDriverUnload();
}

void cDeviceEm84xx::MakePrimaryDevice(bool On) 
{
	Dprintf("%s(%s)\n", __PRETTY_FUNCTION__, On ? "true" : "false");
	new cOsdProviderEm84xx;
}

bool cDeviceEm84xx::SetPlayMode(ePlayMode PlayMode) 
{
	Dprintf("%s(%d)\n", __PRETTY_FUNCTION__, (int)PlayMode);
	switch (PlayMode) {
	case pmNone:
		mThread->Stop();
		break;

	case pmAudioVideo:
	case pmAudioOnlyBlack:
	case pmAudioOnly:
	case pmVideoOnly:
		Open();
		mThread->Start();
		break;

	case pmExtern_THIS_SHOULD_BE_AVOIDED:
		mThread->Stop();
		Close();
		break;

	default:
		break;
	}

	return true;
}

int cDeviceEm84xx::PlayVideo(const uchar *Data, int Length) 
{
	Dprintf("%s(%p,%d)\n", __PRETTY_FUNCTION__, Data, Length);
	//Dprintf("PlayVideo length=%d, first four = %02x %02x %02x %02x\n", Length,
	        //(int)Data[0], (int)Data[1], (int)Data[2], (int)Data[3]);
	return mThread->PlayPacket(Data, Length);
}

int cDeviceEm84xx::PlayAudio(const uchar *Data, int Length) 
{
	Dprintf("%s(%p,%d)\n", __PRETTY_FUNCTION__, Data, Length);
	return mThread->PlayPacket(Data, Length);
}

int64_t cDeviceEm84xx::GetSTC(void) 
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	return FMPGet(FMPI_STC);
}

void cDeviceEm84xx::TrickSpeed(int Speed) 
{
	Dprintf("%s(%d)\n", __PRETTY_FUNCTION__, Speed);
	//Dprintf("TrickMode %lu\n", FMPSet(FMPI_TRICKMODE, FMPV_TRICKMODE_START));
	mThread->TrickSpeed(Speed);
}

void cDeviceEm84xx::Clear(void) 
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
//mThread->Stop();
	FMPFlush(FALSE);
//mThread->Start();

 /*       mThread->Stop();
        Close();
        Open();
        mThread->Start();*/
}

void cDeviceEm84xx::Play(void) 
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	mThread->Play();
}

void cDeviceEm84xx::Freeze(void) 
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	mThread->Freeze();
}

void cDeviceEm84xx::Mute(void) 
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
}

void cDeviceEm84xx::StillPicture(const uchar *Data, int Length) 
{
	Dprintf("%s(%p,%d)\n", __PRETTY_FUNCTION__, Data, Length);
}

bool cDeviceEm84xx::Poll(cPoller &Poller, int TimeoutMs) 
{
	mThread->AddPoller(Poller);
	return Poller.Poll(TimeoutMs);
}

bool cDeviceEm84xx::Flush(int TimeoutMs) 
{
	Dprintf("%s(%d)\n", __PRETTY_FUNCTION__, TimeoutMs);
	FMPFlush(FALSE);
	return true;
}

void cDeviceEm84xx::SetVideoFormat(bool VideoFormat16_9)
{
	FMPSet(FMPI_VIDEOASPECT, VideoFormat16_9 ? FMPV_VIDEOASPECT_16x9_16x9_NORMAL 
	                                         : FMPV_VIDEOASPECT_4x3_4x3_NORMAL);
}

eVideoSystem cDeviceEm84xx::GetVideoSystem(void)
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	DWORD mode = FMPGet(FMPI_VIDEOOUT);
	return (mode & FMPV_VIDEOOUT_STANDARDTV_MASK) == FMPV_VIDEOOUT_NTSC
	       ? vsNTSC 
	       : vsPAL;
}

void cDeviceEm84xx::SetAudioTrackDevice(eTrackType Type)
{
	Dprintf("%s(%d)\n", __PRETTY_FUNCTION__, Type);
}

int cDeviceEm84xx::GetAudioChannelDevice(void)
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
	return 0;
}

void cDeviceEm84xx::SetAudioChannelDevice(int AudioChannel)
{
	Dprintf("%s(%d)\n", __PRETTY_FUNCTION__, AudioChannel);
}

void cDeviceEm84xx::SetVolumeDevice(int Volume)
{
	Dprintf("%s(%d)\n", __PRETTY_FUNCTION__, Volume);
	int fmpvol = Volume * 100 / 255;
	FMPSet(FMPI_LEFT_VOLUME, fmpvol);
	FMPSet(FMPI_RIGHT_VOLUME, fmpvol);
}

void cDeviceEm84xx::SetDigitalAudioDevice(bool On)
{
	Dprintf("%s(%s)\n", __PRETTY_FUNCTION__, On ? "true" : "false");
}

