#include <unistd.h>
#include <vdr/tools.h>
#include "device.h"
#include "em84receiver.h"
#include "common.h"

#define TS_SIZE       188
#define PAY_START     0x40
#define ADAPT_FIELD   0x20
#define PAYLOAD       0x10
#define PTS_DTS_FLAGS 0xC0

cEm84Receiver::cEm84Receiver(cDeviceEm84xx* Em84Device, int Ca, int Vpid, int Apid[], int Dpid[])
:cReceiver(Ca, -1, Vpid, Apid, Dpid, NULL), cThread("femon receiver")
{
  mEm84Device = Em84Device;
  mCurrentAspectRatio = AR_RESERVED;

  mActive = false;
  mVideoPid = Vpid;
  mAudioPid = Apid[0];
  mAC3Pid = Dpid[0];

  mVideoAspectRatio = AR_RESERVED;
}
 
cEm84Receiver::~cEm84Receiver(void)
{
  Detach();
  if (mActive)
  {
     mActive = false;
     Cancel();
  }
}

void cEm84Receiver::SetAspectRatio(int AspectRatio)
{
  if (AspectRatio != mCurrentAspectRatio)
  {
    mCurrentAspectRatio = AspectRatio;
    if (mEm84Device != NULL)
      mEm84Device->SwitchAspectRatio((mCurrentAspectRatio == AR_16_9)?true:false);
  }
}

/* The following function originates from libdvbmpeg: */
void cEm84Receiver::GetVideoInfo(uint8_t *mbuf, int count)
{
  uint8_t *headr;
  int found = 0;
  int c = 0;

  while ((found < 4) && ((c + 4) < count)) {
    uint8_t *b;
    b = mbuf + c;
    if ((b[0] == 0x00) && (b[1] == 0x00) && (b[2] == 0x01) && (b[3] == 0xb3))
       found = 4;
    else
       c++;
    }

  if ((!found) || ((c + 16) >= count)) return;

  headr = mbuf + c + 4;

  int sw = (int)((headr[3] & 0xF0) >> 4);
  switch ( sw ){
    case 1:
      mVideoAspectRatio = AR_1_1;
      break;
    case 2:
      mVideoAspectRatio = AR_4_3;
      break;
    case 3:
      mVideoAspectRatio = AR_16_9;
      break;
    case 4:
      mVideoAspectRatio = AR_2_21_1;
      break;
    case 5 ... 15:
    default:
      mVideoAspectRatio = AR_RESERVED;
      break;
  }

  SetAspectRatio(mVideoAspectRatio);
}

void cEm84Receiver::Activate(bool On)
{
  if (On)
  {
     if (!mActive)
        Start();
  }
  else if (mActive)
  {
     mActive = false;
     Cancel();
  }
}

void cEm84Receiver::Receive(uchar *Data, int Length)
{
  // TS packet length: TS_SIZE
  if (Length == TS_SIZE)
  {
     int pid = ((Data[1] & 0x1f) << 8) | (Data[2]);
 
     /* the following originates from libdvbmpeg: */
     if (!(Data[3] & PAYLOAD))
     {
        return;
     }

     uint8_t off = 0;
     if (Data[3] & ADAPT_FIELD)
     {
        off = Data[4] + 1;
     }
     if (Data[1] & PAY_START)
     {
        uint8_t *sb = Data + 4 + off;
        if (sb[7] & PTS_DTS_FLAGS) {
           uint8_t *pay = sb + sb[8] + 9; 
           int l = TS_SIZE - 13 - off - sb[8];
           if (pid == mVideoPid) {
              GetVideoInfo(pay, l);
           }
        }
     }
     /* end */
   }
}

void cEm84Receiver::Action(void)
{
  cTimeMs t;
  mActive = true;
  while (mActive)
  {
        t.Set(0);
        // TS packet 188 bytes - 4 byte header; MPEG standard defines 1Mbit = 1000000bit
	 cCondWait::SleepMs(100);
  }
}

