#include "osd.h"
#include "common.h"
#include <string.h>

typedef unsigned char *PBYTE;
#include <fmp.h>
#include <rm84mpegcmn.h>
#include <hwiface.h>

//#define EM84_HAS_4MB	1

#ifdef EM84_HAS_4MB
static int EM84OSD_WIDTH = 720;
static int EM84OSD_HEIGHT = 576;
#else
static int EM84OSD_WIDTH = 300;
static int EM84OSD_HEIGHT = 130;
#endif

typedef struct _tag_em84_paletteentry
	{
		unsigned char a;
		unsigned char r;
		unsigned char g;
		unsigned char b;
	} EM84PALENTRY;

class cOsdEm84xx: public cOsd {
private:
	
	static IOsd *mOsd;
	static unsigned char mOsdBuffer[720*576+1032];
	static EM84PALENTRY mPaletteEM84[256];
	static int 	mUsedColors;

protected:
	void setOSDPixel(int x, int y, unsigned char color);
	void resetOSD();
	void resetPalette();
	void displayOSD(int x, int y);
	void commitPaletteToOsd();
	void setOSDPaletteEntry(int palidx, unsigned char r,unsigned char g,unsigned char b,unsigned char a);
	int suiteColorToPalette(unsigned int vdrcolor);
	static void rgbtoyuv(double R,double G,double B,double *y,double *u,double *v);

public:
	cOsdEm84xx(int Left, int Top);
	virtual ~cOsdEm84xx();

	virtual eOsdError CanHandleAreas(const tArea *Areas, int NumAreas);
	virtual eOsdError SetAreas(const tArea *Areas, int NumAreas);
	virtual void Flush(void);
};

IOsd *cOsdEm84xx::mOsd = NULL;
unsigned char cOsdEm84xx::mOsdBuffer[720*576+1032];
EM84PALENTRY cOsdEm84xx::mPaletteEM84[256];
int cOsdEm84xx::mUsedColors = 0;


cOsdEm84xx::cOsdEm84xx(int Left, int Top):
		cOsd(Left, Top)
{
	if ((mOsd == NULL) && (FMPQueryInterface(IID_IOSD, (void**)&mOsd)))
	{
		Dprintf("could not open OSD\n");
		mOsd = NULL;
  	}

	resetOSD();
	resetPalette();
}

cOsdEm84xx::~cOsdEm84xx()
{
	if (mOsd)
		mOsd->lpVtbl->Off(mOsd);
		//mOsd->lpVtbl->Flush(mOsd);	
}

eOsdError cOsdEm84xx::CanHandleAreas(const tArea *Areas, int NumAreas)
{
#ifndef EM84_HAS_4MB
	eOsdError res;
	if (NumAreas == 1 && (res = cOsd::CanHandleAreas(Areas, NumAreas)) == oeOk) {
		int w = Areas[0].x2 - Areas[0].x1 + 1;
		int h = Areas[0].y2 - Areas[0].y1 + 1;
		return w * h < 39000 ? oeOk : oeOutOfMemory;
	}
	return res == oeOk ? oeTooManyAreas : res;
#else
	return oeOk;
#endif
}

eOsdError cOsdEm84xx::SetAreas(const tArea *Areas, int NumAreas)
{
#ifndef EM84_HAS_4MB
	eOsdError res;
	if ((res = CanHandleAreas(Areas, NumAreas)) == oeOk) {
		EM84OSD_WIDTH = Areas[0].x2 - Areas[0].x1 + 1;
		EM84OSD_HEIGHT = Areas[0].y2 - Areas[0].y1 + 1;
		resetOSD();
		resetPalette();
		cOsd::SetAreas(Areas, NumAreas);
	}
	return res;
#else
	return oeOk;
#endif
}

void cOsdEm84xx::Flush(void)
{	
	cBitmap *Bitmap;

	mOsd->lpVtbl->Flush(mOsd);

	if (!mOsd)
		return;

	// first build a common palette (don't know if this will work !?!?
	//resetOSD();
	//resetPalette();
	for (int i = 0; (Bitmap = GetBitmap(i)) != NULL; i++)
	{
		int x1 = 0, y1 = 0, x2 = 0, y2 = 0;
		if (Bitmap->Dirty(x1, y1, x2, y2))
		{
			// commit colors:
	       	int NumColors;
		       const tColor *Colors = Bitmap->Colors(NumColors);
       		if (Colors)
			{
				int* newcolors = (int*)malloc(NumColors*sizeof(int));
				for (int i = 0; i < NumColors; i++)
				{
					newcolors[i] = suiteColorToPalette(Colors[i]);
				}
			
				// draw window to osd!
				int x,y;
				for (y = y1; y <= y2; y++)
				{
					for (x = x1; x <= x2; x++)
					{
						setOSDPixel(Bitmap->X0()+x, Bitmap->Y0()+y, newcolors[*((unsigned char*)(Bitmap->Data(x,y)))]);
					}
				}
			}
		}
	}

	
	commitPaletteToOsd();
	displayOSD(Left(), Top());
}

int cOsdEm84xx::suiteColorToPalette(unsigned int vdrcolor)
{
	int difa, difr, difg, difb;
	int alldif;
	int i;
	int found = -1;
	for (i=1; i <= mUsedColors; i++)
	{
		if 	(
			(mPaletteEM84[i].a == ((vdrcolor & 0xFF000000) >> 24))
			&&
			(mPaletteEM84[i].b == (vdrcolor & 0x000000FF))
			&&
			(mPaletteEM84[i].g == ((vdrcolor & 0x00FF00) >> 8))
			&&
			(mPaletteEM84[i].r == ((vdrcolor & 0x00FF0000) >> 16))
			)
		{
			found = i;
			break;
		}
	}

	if (found != -1)
		return found;

	if (mUsedColors < 255)
	{
		mUsedColors++;
		mPaletteEM84[mUsedColors].a = ((vdrcolor & 0xFF000000) >> 24);
		mPaletteEM84[mUsedColors].r = ((vdrcolor & 0x00FF0000) >> 16);
		mPaletteEM84[mUsedColors].g = ((vdrcolor & 0x00FF00) >> 8);
		mPaletteEM84[mUsedColors].b = (vdrcolor & 0x000000FF);

		return mUsedColors;
	}

	// choose best one! ;)
	found = -1;
	alldif = 4*256;
	for (i=1; i <= mUsedColors; i++)
	{
		difa = mPaletteEM84[i].a - ((vdrcolor & 0xFF000000) >> 24);
		difr = mPaletteEM84[i].r - ((vdrcolor & 0x00FF0000) >> 16);
		difg = mPaletteEM84[i].g - ((vdrcolor & 0x00FF00) >> 8);
		difb = mPaletteEM84[i].b - (vdrcolor & 0x000000FF);
		if (difa < 0)
			difa = -1*difa;
		if (difr < 0)
			difr = -1*difr;
		if (difg < 0)
			difg = -1*difg;
		if (difb < 0)
			difb = -1*difb;		

		if ((difa + difr + difg + difb) < alldif)
		{
			alldif = (difa + difr + difg + difb);
			found = i;
		}
	}
	return found;
}

void cOsdEm84xx::resetOSD()
{
	int size;

	memset(mOsdBuffer,0x00,(EM84OSD_WIDTH*EM84OSD_HEIGHT+1032));

	size = 1032+(EM84OSD_WIDTH*EM84OSD_HEIGHT);
	mOsdBuffer[0] = 0x3e;
	mOsdBuffer[1]= size >> 16;
	mOsdBuffer[2]= (size >> 8) & 0xff;
	mOsdBuffer[3]= size & 0xff;
	mOsdBuffer[4]= EM84OSD_WIDTH >> 8;
	mOsdBuffer[5]= EM84OSD_WIDTH & 0xff;
	mOsdBuffer[6]= EM84OSD_HEIGHT >> 8;
	mOsdBuffer[7]= EM84OSD_HEIGHT & 0xff;
}

void cOsdEm84xx::resetPalette()
{
	int i;
	for (i=0; i < 256; i++)
	{
		mPaletteEM84[i].a = 0x00;
		mPaletteEM84[i].r = 0x00;
		mPaletteEM84[i].g = 0x00;
		mPaletteEM84[i].b = 0x00;
	}
	mUsedColors = 0;
}

void cOsdEm84xx::commitPaletteToOsd()
{
	int i;
	for (i=0; i < 256; i++)
	{
		setOSDPaletteEntry(i, mPaletteEM84[i].r,mPaletteEM84[i].g,mPaletteEM84[i].b,mPaletteEM84[i].a);
	}
}

void cOsdEm84xx::displayOSD(int x, int y)
{
  if (mOsd )
  {
	int size = 1032+(EM84OSD_WIDTH*EM84OSD_HEIGHT);
       OSD_WND Wnd;
	Wnd.x = x;
	Wnd.y = y;
	Wnd.w = EM84OSD_WIDTH;
	Wnd.h = EM84OSD_HEIGHT;

	mOsd->lpVtbl->Write(mOsd , mOsdBuffer, size);
	mOsd->lpVtbl->SetDest(mOsd , &Wnd);
	//OSSleep(10);
	mOsd->lpVtbl->On(mOsd );
   }  
}


void cOsdEm84xx::setOSDPixel(int x, int y, unsigned char color)
{
	if (x >= EM84OSD_WIDTH)
		return;
	if (y >= EM84OSD_HEIGHT)
		return;

	mOsdBuffer[1032+(y*EM84OSD_WIDTH+x)] = color;
}

void cOsdEm84xx::setOSDPaletteEntry(int palidx, unsigned char r,unsigned char g,unsigned char b,unsigned char a)
{
	unsigned char* ptr = &mOsdBuffer[8+(palidx*4)];
	unsigned char temp[3];
	double R,G,B,y,u,v;
		
	R=r;
	G=g;
	B=b;
	rgbtoyuv(R,G,B,&y,&u,&v);
	temp[0] = (unsigned char) y;
	temp[1] = (unsigned char) u;
	temp[2] = (unsigned char) v;

	*ptr++ = a;
	*ptr++ = temp[0];
	*ptr++ = temp[1];
	*ptr++ = temp[2];
}

void cOsdEm84xx::rgbtoyuv(double R,double G,double B,double *y,double *u,double *v)
{
	*y=(0.257*R + 0.504*G + 0.098*B) + 16.0;
	*u=(-0.148*R - 0.291*G + 0.439*B) + 128.0;
	*v=(0.439*R - 0.368*G - 0.071*B) + 128.0;
}

cOsdProviderEm84xx::cOsdProviderEm84xx(void)
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
}

cOsd *cOsdProviderEm84xx::CreateOsd(int Left, int Top)
{
	Dprintf("%s(%d,%d)\n", __PRETTY_FUNCTION__, Left, Top);
	return new cOsdEm84xx(Left, Top);
}
