#include "thread.h"
#include "common.h"
#include <unistd.h>

cThreadEm84xx::cThreadEm84xx():
		cThread("Em84xx replay"),
		mActive(false),
		mTrickMode(false),
		mInTrick(false)
{
}

cThreadEm84xx::~cThreadEm84xx()
{
	Stop();
}

void cThreadEm84xx::Start(void)
{
	if (pipe(mPipe) != 0) {
		LOG_ERROR;
		return;
	}
	fcntl(mPipe[1], F_SETFL, O_NONBLOCK);
	cThread::Start();
}

void cThreadEm84xx::Stop(void)
{
	if (mActive) {
		mActive = false;
		close(mPipe[1]);
		Cancel(3);
	}
}

void cThreadEm84xx::Action(void)
{
	mActive = true;
	FMPPlay();
	while (mActive) {
		DWORD rc;

		if ((rc = FMPGetBuffer(&mBuffer, TRUE)) != FMPE_OK) {
			esyslog("Couldn't allocate em8400 replaybuffer: %s", FMPGetLastError(rc));
			Dprintf("Couldn't allocate em8400 replaybuffer: %s\n", FMPGetLastError(rc));
			break;
		}
		mBuffer.dwDataSize = 0;
		//mBuffer.dwFlags = mTrickMode ? FMP_TRICKMODE_SAMPLE : 0;
		//mBuffer.dwFlagsEx = FMP_VIDEO_PES | FMP_AUDIO_PES | FMP_DVD_AUDIO_PES;

		while (mActive && mBuffer.dwDataSize < mBuffer.dwBufferSize) {
			int r = 0;
			while (mActive && (r = read(mPipe[0], mBuffer.pBuffer + mBuffer.dwDataSize, 
			                            mBuffer.dwBufferSize - mBuffer.dwDataSize)) == -1 &&
				   errno == EINTR)
				;
		
			if (r == -1) {
				if (mActive)
					LOG_ERROR;
			} else 
				mBuffer.dwDataSize += r;
		}

		//Dprintf("%d to driver\n", (int)mBuffer.dwDataSize);
		FMPPush(&mBuffer);
	}
	FMPStop();
	close(mPipe[0]);
	mActive = false;
}

int cThreadEm84xx::PlayPacket(const uchar *Data, int Length)
{
	//Dprintf("%s\n", __PRETTY_FUNCTION__);
	//return safe_write(mPipe[1], Data, Length);
	return WriteAllOrNothing(mPipe[1], Data, Length, 1000, 10);
}

void cThreadEm84xx::TrickSpeed(int Speed)
{
	mTrickMode = Speed > 0;
}

void cThreadEm84xx::AddPoller(cPoller &Poller)
{
	Poller.Add(mPipe[1], true);
}

