#include "thread.h"
#include "common.h"
#include <unistd.h>

cThreadEm84xx::cThreadEm84xx():
		cThread("Em84xx replay"),
		mActive(false),
		mTrickMode(false),
		mInTrick(false)
{
}

cThreadEm84xx::~cThreadEm84xx()
{
	Stop();
}

void cThreadEm84xx::Start(void)
{
	if (pipe(mPipe) != 0) {
		LOG_ERROR;
		return;
	}
	fcntl(mPipe[1], F_SETFL, O_NONBLOCK);
	cThread::Start();
}

void cThreadEm84xx::Stop(void)
{
	if (mActive) {
		mActive = false;
		close(mPipe[1]);
		Cancel(3);
	}
}

void cThreadEm84xx::Action(void)
{
	uchar buffer[BUFSIZ];
	int used = 0;

	mActive = true;
	FMPPlay();
	while (mActive) {
		int r;
		while (mActive && (r = read(mPipe[0], buffer + used, sizeof(buffer) - used)) == -1 &&
		       errno == EINTR)
			;
		if (r == -1) {
			if (mActive) LOG_ERROR;
			break;
		}
		used += r;

		if (used < 10)
			continue;

		DWORD rc;
		FMP_BUFFER fmpbuf;
		uchar id = buffer[3];
		int len;
		fmpbuf.dwFlagsEx = 0;
		switch (id) {
		default:
		case 0x00 ... 0xB8: // video stream start codes
		case 0xB9: // Program end
		case 0xBC: // Programm stream map
		case 0xF0 ... 0xFF: // reserved
			 len = 6;
			 break;

		case 0xBA: // Pack header
			 if ((buffer[4] & 0xC0) == 0x40) // MPEG2
				len = 14;
			 // to be absolutely correct we would have to add the stuffing bytes
			 // as well, but at this point we only may have 6 bytes of data avail-
			 // able. So it's up to the higher level to resync...
			 //return 14 + (data[13] & 0x07); // add stuffing bytes
			 else // MPEG1
				len = 12;
			break;

		case 0xBB: // System header
		case 0xBD: // Private stream1
		case 0xBE: // Padding stream
		case 0xBF: // Private stream2 (navigation data)
			 len = 6 + buffer[4] * 256 + buffer[5];
			 break;

		case 0xC0 ... 0xCF: // all the rest (the real packets)
		case 0xD0 ... 0xDF:
			 len = 6 + buffer[4] * 256 + buffer[5];
			 fmpbuf.dwFlagsEx = FMP_DVD_AUDIO_PES;
			 break;

		case 0xE0 ... 0xEF:
			 fmpbuf.dwFlagsEx = FMP_VIDEO_PES;
			 len = 6 + buffer[4] * 256 + buffer[5];
			 break;
		}

		if (len > used) // partial packet
			continue;

		//Dprintf("found pes packet, id=%x, len=%d, flag=%d\n", (int)id, len, fmpbuf.dwFlagsEx);

		if (fmpbuf.dwFlagsEx != 0) {
			if ((rc = FMPGetBuffer(&fmpbuf, TRUE)) != FMPE_OK) {
				esyslog("Couldn't allocate em8400 replaybuffer: %s", FMPGetLastError(rc));
				break;
			}
			//Dprintf("flags after get %lu %lu\n", fmpbuf.dwFlags, fmpbuf.dwFlagsEx);
			fmpbuf.dwDataSize = 12 + len;
			fmpbuf.pBuffer[0] = 0x00;
			fmpbuf.pBuffer[1] = 0x00;
			fmpbuf.pBuffer[2] = 0x01;
			fmpbuf.pBuffer[3] = 0xBA;
			fmpbuf.pBuffer[4] = 0x00;
			fmpbuf.pBuffer[5] = 0x00;
			fmpbuf.pBuffer[6] = 0x00;
			fmpbuf.pBuffer[7] = 0x00;
			fmpbuf.pBuffer[8] = ((len) & 0xff000000) >> 24;
			fmpbuf.pBuffer[9] = ((len) & 0xff0000) >> 16;
			fmpbuf.pBuffer[10] = ((len)  & 0xff00) >> 8;
			fmpbuf.pBuffer[11] = ((len) & 0xff);
			memcpy(fmpbuf.pBuffer + 12, buffer, len);
			FMPPush(&fmpbuf);
		}

		used -= len;
		memmove(buffer, buffer + len, used);
/*
		while (mActive && mBuffer.dwDataSize < mBuffer.dwBufferSize) {
			int r = 0;
			while (mActive && (r = read(mPipe[0], mBuffer.pBuffer + mBuffer.dwDataSize, 
			                            mBuffer.dwBufferSize - mBuffer.dwDataSize)) == -1 &&
				   errno == EINTR)
				;
		
			if (r == -1) {
				if (mActive)
					LOG_ERROR;
			} else 
				mBuffer.dwDataSize += r;
		}
*/
		//Dprintf("%d to driver\n", (int)mBuffer.dwDataSize);
	}
	FMPStop();
	close(mPipe[0]);
	mActive = false;
}

int cThreadEm84xx::PlayPacket(const uchar *Data, int Length)
{
	//Dprintf("%s\n", __PRETTY_FUNCTION__);
	//return safe_write(mPipe[1], Data, Length);
	return WriteAllOrNothing(mPipe[1], Data, Length, 1000, 10);
}

void cThreadEm84xx::TrickSpeed(int Speed)
{
	mTrickMode = Speed > 0;
}

void cThreadEm84xx::AddPoller(cPoller &Poller)
{
	Poller.Add(mPipe[1], true);
}

