#include "osd.h"
#include "common.h"
#include <string.h>

typedef unsigned char *PBYTE;
#include <fmp.h>
#include <rm84mpegcmn.h>
#include <hwiface.h>

//#define EM84_HAS_4MB	1

//static int EM84OSD_WIDTH = 500;
//static int EM84OSD_HEIGHT = 312;

#ifndef EM84_HAS_4MB
#define EM84OSD_TOTAL_BUFFERSIZE	41000
#define EM84OSD_ALTER_WIDTH		500
#define EM84OSD_ALTER_HEIGHT	320
#define EM84OSD_PALETTESIZE		8
#else
#define EM84OSD_TOTAL_BUFFERSIZE	(720*576+1032)
#define EM84OSD_ALTER_WIDTH		720
#define EM84OSD_ALTER_HEIGHT	576
#define EM84OSD_PALETTESIZE		256
#endif

typedef struct _tag_em84_paletteentry
{
	unsigned char a;
	unsigned char r;
	unsigned char g;
	unsigned char b;
} EM84PALENTRY;

class cOsdEm84xx: public cOsd {
private:
	
	static IOsd *mOsd;
	static unsigned char mOsdBuffer[EM84OSD_TOTAL_BUFFERSIZE];
	static EM84PALENTRY mPaletteEM84[EM84OSD_PALETTESIZE];
	static int 	mUsedColors;
	static int	mUsedColorsHilite;

	int mCurwidth;
	int mCurheight;
	bool mUsealt;

	int mContainerArea;
	int mContainedArea;
	tArea mHiliteArea;
	tArea mHiliteAreaDest;
	tArea mAllArea;

	bool AreaContainedByAread(const tArea AreaContains, const tArea AreaIsContained);

protected:
	void setOSDPixel(int x, int y, unsigned char color);
	void resetOSD();
	void resetPalette();
	void displayOSD(int x, int y);
	void commitPaletteToOsd();
	void setOSDPaletteEntry(int palidx, unsigned char r,unsigned char g,unsigned char b,unsigned char a);
	int suiteColorToPalette(unsigned int vdrcolor, bool hilitepalette=false);
	static void rgbtoyuv(double R,double G,double B,double *y,double *u,double *v);

public:
	cOsdEm84xx(int Left, int Top);
	virtual ~cOsdEm84xx();

	virtual eOsdError CanHandleAreas(const tArea *Areas, int NumAreas);
	//virtual eOsdError SetAreas(const tArea *Areas, int NumAreas);
	virtual void Flush(void);
};

IOsd *cOsdEm84xx::mOsd = NULL;
unsigned char cOsdEm84xx::mOsdBuffer[EM84OSD_TOTAL_BUFFERSIZE];
EM84PALENTRY cOsdEm84xx::mPaletteEM84[EM84OSD_PALETTESIZE];
int cOsdEm84xx::mUsedColors = 0;
int cOsdEm84xx::mUsedColorsHilite = 0;


cOsdEm84xx::cOsdEm84xx(int Left, int Top):
		cOsd(Left, Top)
{
Dprintf("%s\n", __PRETTY_FUNCTION__);

	mCurwidth = -1;
	mCurheight = -1;
	mUsealt = false;
	mContainerArea = -1;
	mContainedArea = -1;

	if ((mOsd == NULL) && (FMPQueryInterface(IID_IOSD, (void**)&mOsd)))
	{
		Dprintf("could not open OSD\n");
		mOsd = NULL;
  	}
}

cOsdEm84xx::~cOsdEm84xx()
{
	if (mOsd)
		mOsd->lpVtbl->Off(mOsd);
		//mOsd->lpVtbl->Flush(mOsd);	
}

bool cOsdEm84xx::AreaContainedByAread(const tArea AreaContains, const tArea AreaIsContained)
{
  if ((AreaIsContained.x1 >= AreaContains.x1)
     &&
     (AreaIsContained.y1 >= AreaContains.y1)
     &&
     (AreaIsContained.x2 <= AreaContains.x2)
     &&
     (AreaIsContained.y2 <= AreaContains.y2))
    return true;

  return false;
}

eOsdError cOsdEm84xx::CanHandleAreas(const tArea *Areas, int NumAreas)
{
Dprintf("%s\n", __PRETTY_FUNCTION__);

#ifndef EM84_HAS_4MB
  int totalwidth,totalheight;
  int osdsize;

  if ((NumAreas != 1) && (NumAreas != 3))
    return oeTooManyAreas;

  for (int i = 0; i < NumAreas; i++)
  {
    if (Areas[i].bpp != 2)
	return oeTooManyColors;
  }

  if (NumAreas == 3)
  {
    if (AreaContainedByAread(Areas[0], Areas[1]))
      return oeWrongAlignment;
    else if (AreaContainedByAread(Areas[1], Areas[0]))
      return oeWrongAlignment;
    
    mContainerArea = 0;
    mContainedArea = 1;
      

    totalwidth  = Areas[mContainerArea].x2 - Areas[mContainerArea].x1 + 1;
    totalheight = Areas[mContainerArea].y2 - Areas[mContainerArea].y1 + 1;

    mHiliteArea.x1 = Areas[mContainedArea].x1;
    mHiliteArea.x2 = Areas[mContainedArea].x2;
    mHiliteArea.y1 = Areas[mContainedArea].y1;
    mHiliteArea.y2 = Areas[mContainedArea].y2;

    mHiliteAreaDest.x1 = Areas[2].x1;
    mHiliteAreaDest.x2 = Areas[2].x2;
    mHiliteAreaDest.y1 = Areas[2].y1;
    mHiliteAreaDest.y2 = Areas[2].y2;


    mAllArea.x1 = Areas[mContainerArea].x1;
    mAllArea.x2 = Areas[mContainerArea].x2;
    mAllArea.y1 = Areas[mContainerArea].y1;
    mAllArea.y2 = Areas[mContainerArea].y2;

    totalwidth  = Areas[0].x2 - Areas[0].x1 + 1;
    totalheight = Areas[0].y2 - Areas[0].y1 + 1;
  }
  else
  {
    totalwidth  = Areas[0].x2 - Areas[0].x1 + 1;
    totalheight = Areas[0].y2 - Areas[0].y1 + 1;
  }

  osdsize = totalwidth / 4;
  if ((totalwidth % 4) > 0)
    osdsize++;
  osdsize *= totalheight;

  if (osdsize <= 40000)
  {
    mCurwidth = totalwidth;
    mCurheight = totalheight;
  }
  else
  {
    mUsealt = true;

    mCurwidth = EM84OSD_ALTER_WIDTH;
    mCurheight = EM84OSD_ALTER_HEIGHT;
  }

  resetOSD();
  resetPalette();

  return oeOk;

#else
  /*eOsdError Result = cOsd::CanHandleAreas(Areas, NumAreas);
  if (Result == oeOk)
  {
	int minx = 10000;
	int miny = 10000;
	int maxx = -1;
	int maxy = -1;
	int totalwidth,totalheight;
	int osdsize;

	for (int i = 0; i < NumAreas; i++)
	{
         if (Areas[i].x1 < minx)
		minx = Areas[i].x1;
	  if (Areas[i].y1 < miny)
		miny = Areas[i].y1;
	  if (Areas[i].x2 > maxx)
		maxx = Areas[i].x2;
	  if (Areas[i].y2 > maxy)
		maxy = Areas[i].y2;
	}

	totalwidth = maxx - minx + 1;
	totalheight = maxy - miny + 1;


	mCurwidth = totalwidth;
	mCurheight = totalheight;

Dprintf("Habe hier eine grsse von %d-%d\n",mCurwidth,mCurheight);

	if (mCurwidth > EM84OSD_ALTER_WIDTH)
 		mCurwidth = EM84OSD_ALTER_WIDTH;
	if (mCurheight > EM84OSD_ALTER_HEIGHT)
		mCurheight = EM84OSD_ALTER_HEIGHT;
  

	resetOSD();
	resetPalette();

	return oeOk;
  }
  return Result;*/

  /*eOsdError Result = cOsd::CanHandleAreas(Areas, NumAreas);
  if (Result == oeOk)
  {*/
	int minx = 10000;
	int miny = 10000;
	int maxx = -1;
	int maxy = -1;
	int totalwidth,totalheight;
	int osdsize;

	for (int i = 0; i < NumAreas; i++)
	{
         if (Areas[i].x1 < minx)
		minx = Areas[i].x1;
	  if (Areas[i].y1 < miny)
		miny = Areas[i].y1;
	  if (Areas[i].x2 > maxx)
		maxx = Areas[i].x2;
	  if (Areas[i].y2 > maxy)
		maxy = Areas[i].y2;
	}

	totalwidth = maxx - minx + 1;
	totalheight = maxy - miny + 1;

	//mCurwidth = totalwidth;
	//mCurheight = totalheight;
Dprintf("set to %d,%d\n",mCurwidth,mCurheight);
	mCurwidth = EM84OSD_ALTER_WIDTH;
	mCurheight = EM84OSD_ALTER_HEIGHT;

	/*if (mCurwidth > EM84OSD_ALTER_WIDTH)
 		mCurwidth = EM84OSD_ALTER_WIDTH;
	if (mCurheight > EM84OSD_ALTER_HEIGHT)
		mCurheight = EM84OSD_ALTER_HEIGHT;*/


// MAKE complete area for 4MB durchsichtig ;)
	resetOSD();
	resetPalette();
	suiteColorToPalette(0x00000000);

	return oeOk;
  /*}
  return Result;*/

#endif
}

void cOsdEm84xx::Flush(void)
{	
Dprintf("%s\n", __PRETTY_FUNCTION__);

	/*int baseX = 10000;
	int baseY = 10000;*/

	cBitmap *Bitmap;

	if (!mOsd)
		return;

	mOsd->lpVtbl->Flush(mOsd);



	// first build a common palette (don't know if this will work !?!?
	for (int i = 0; (Bitmap = GetBitmap(i)) != NULL; i++)
	{
		/*if (Bitmap->X0() < baseX)
			baseX = Bitmap->X0();
		if (Bitmap->Y0() < baseY)
			baseY = Bitmap->Y0();*/

		int x1 = 0, y1 = 0, x2 = 0, y2 = 0;
		if (Bitmap->Dirty(x1, y1, x2, y2))
		{
			// commit colors:
	       	int NumColors;
		       const tColor *Colors = Bitmap->Colors(NumColors);
       		if (Colors)
			{
				int* newcolors = (int*)malloc(NumColors*sizeof(int));
				for (int j = 0; j < NumColors; j++)
				{
					if (mContainedArea == i)
						newcolors[j] = suiteColorToPalette(Colors[j],true);
					else
						newcolors[j] = suiteColorToPalette(Colors[j]);
				}
			
				// draw window to osd!
				int x,y;
				for (y = y1; y <= y2; y++)
				{
					for (x = x1; x <= x2; x++)
					{
						setOSDPixel(Bitmap->X0()+x, Bitmap->Y0()+y, newcolors[*((unsigned char*)(Bitmap->Data(x,y)))]);
					}
				}
			}
		}
	}

	
	commitPaletteToOsd();
	/*if ((baseX != 10000) && (baseY != 10000))
		displayOSD(Left()+baseX, Top()+baseY);
	else*/
		displayOSD(Left(), Top());
}

int cOsdEm84xx::suiteColorToPalette(unsigned int vdrcolor, bool hilitepalette /*=false*/)
{
	int difr, difg, difb;
	int alldif;
	int i;
	int found = -1;
	int start = 0;
	int stop = mUsedColors;

	if (hilitepalette)
	{
		start += 4;
		stop = mUsedColorsHilite + 4;
	}

	for (i=start ; i < stop; i++)
	{
		if 	(
			(mPaletteEM84[i].a == ((vdrcolor & 0xFF000000) >> 24))
			&&
			(mPaletteEM84[i].b == (vdrcolor & 0x000000FF))
			&&
			(mPaletteEM84[i].g == ((vdrcolor & 0x00FF00) >> 8))
			&&
			(mPaletteEM84[i].r == ((vdrcolor & 0x00FF0000) >> 16))
			)
		{
			found = i;
			break;
		}
	}

	if (found != -1)
	{
		if (!hilitepalette)
			return found;
		else
			return (found-4);
	}

#ifndef EM84_HAS_4MB
	if ((!hilitepalette) && (mUsedColors < 4))
	{
		mPaletteEM84[mUsedColors].a = ((vdrcolor & 0xFF000000) >> 24);
		mPaletteEM84[mUsedColors].r = ((vdrcolor & 0x00FF0000) >> 16);
		mPaletteEM84[mUsedColors].g = ((vdrcolor & 0x00FF00) >> 8);
		mPaletteEM84[mUsedColors].b = (vdrcolor & 0x000000FF);
		mUsedColors++;
		return (mUsedColors-1);
	}
	else if ((hilitepalette) && (mUsedColorsHilite < 4))
	{
		mPaletteEM84[mUsedColorsHilite+4].a = ((vdrcolor & 0xFF000000) >> 24);
		mPaletteEM84[mUsedColorsHilite+4].r = ((vdrcolor & 0x00FF0000) >> 16);
		mPaletteEM84[mUsedColorsHilite+4].g = ((vdrcolor & 0x00FF00) >> 8);
		mPaletteEM84[mUsedColorsHilite+4].b = (vdrcolor & 0x000000FF);
		mUsedColorsHilite++;
		return (mUsedColorsHilite-1);
	}
#else
	if (mUsedColors < 256)
	{
		mPaletteEM84[mUsedColors].a = ((vdrcolor & 0xFF000000) >> 24);
		mPaletteEM84[mUsedColors].r = ((vdrcolor & 0x00FF0000) >> 16);
		mPaletteEM84[mUsedColors].g = ((vdrcolor & 0x00FF00) >> 8);
		mPaletteEM84[mUsedColors].b = (vdrcolor & 0x000000FF);
		mUsedColors++;
		return (mUsedColors-1);
	}
#endif

	// choose best one! ;)
	found = -1;
	alldif = 3*256;

#ifndef EM84_HAS_4MB
	start = 0;
	stop  = 4;
	if (hilitepalette)
	{
		start = 4;
		stop  = 8;
	}
#else
	start = 0;
	stop  = 256;
#endif

	for (i=start; i < stop; i++)
	{
		difr = mPaletteEM84[i].r - ((vdrcolor & 0x00FF0000) >> 16);
		difg = mPaletteEM84[i].g - ((vdrcolor & 0x00FF00) >> 8);
		difb = mPaletteEM84[i].b - (vdrcolor & 0x000000FF);
		if (difr < 0)
			difr = -1*difr;
		if (difg < 0)
			difg = -1*difg;
		if (difb < 0)
			difb = -1*difb;		

		if ((difr + difg + difb) < alldif)
		{
			alldif = (difr + difg + difb);
			found = i;
		}
	}
	if (!hilitepalette)
		return found;
	else
		return (found-4);
}

void cOsdEm84xx::resetOSD()
{
	int size;
	memset(mOsdBuffer,0x00,EM84OSD_TOTAL_BUFFERSIZE);

	// calc. osd-size from width+height
#ifndef EM84_HAS_4MB
	size= mCurwidth / 4;
	if ((mCurwidth % 4) > 0)
		size++;
	size *= mCurheight;
	size += 40;
#else
	size = mCurwidth*mCurheight+1032;
#endif

#ifndef EM84_HAS_4MB
	mOsdBuffer[0] = 0x10;
#else
	mOsdBuffer[0] = 0x3e;
#endif
	mOsdBuffer[1]= size >> 16;
	mOsdBuffer[2]= (size >> 8) & 0xff;
	mOsdBuffer[3]= size & 0xff;
	mOsdBuffer[4]= mCurwidth >> 8;
	mOsdBuffer[5]= mCurwidth & 0xff;
	mOsdBuffer[6]= mCurheight >> 8;
	mOsdBuffer[7]= mCurheight & 0xff;
}

void cOsdEm84xx::resetPalette()
{

	for (int i=0; i < EM84OSD_PALETTESIZE; i++)
	{
		mPaletteEM84[i].a = 0x00;
		mPaletteEM84[i].r = 0x00;
		mPaletteEM84[i].g = 0x00;
		mPaletteEM84[i].b = 0x00;
	}
	mUsedColors = 0;
	mUsedColorsHilite = 0;
}

void cOsdEm84xx::commitPaletteToOsd()
{
	int i;
	for (i=0; i < EM84OSD_PALETTESIZE; i++)
	{
		setOSDPaletteEntry(i, mPaletteEM84[i].r,mPaletteEM84[i].g,mPaletteEM84[i].b,mPaletteEM84[i].a);
	}
}

void cOsdEm84xx::displayOSD(int x, int y)
{
Dprintf("%s(%d,%d)\n", __PRETTY_FUNCTION__, x, y);

  if (mOsd )
  {
	int size;
#ifndef EM84_HAS_4MB
	size= mCurwidth / 4;
	if ((mCurwidth % 4) > 0)
		size++;
	size *= mCurheight;
	size += 40;
#else
	size = mCurwidth*mCurheight+1032;
#endif
	OSD_WND WndHilite;
	if (mContainedArea > -1)
	{
	  WndHilite.x = mHiliteAreaDest.x1 - mAllArea.x1;
	  WndHilite.y = mHiliteAreaDest.y1 - mAllArea.y1;
	  WndHilite.w = mHiliteArea.x2 - mHiliteArea.x1;
	  WndHilite.h = mHiliteArea.y2 - mHiliteArea.y1;
	}
	else
	{
	  WndHilite.x = 0;
	  WndHilite.y = 0;
	  WndHilite.w = 0;
	  WndHilite.h = 0;
	}
	mOsd->lpVtbl->SetHli(mOsd, &WndHilite);
       

       OSD_WND Wnd;
	Wnd.x = x;
	Wnd.y = y;
	Wnd.w = mCurwidth;
	Wnd.h = mCurheight;

	mOsd->lpVtbl->Write(mOsd , mOsdBuffer, size);
	mOsd->lpVtbl->SetDest(mOsd , &Wnd);

	//OSSleep(10);
	mOsd->lpVtbl->On(mOsd );
   }  
}


void cOsdEm84xx::setOSDPixel(int x, int y, unsigned char color)
{
#ifndef EM84_HAS_4MB

	if (mContainedArea > -1)
	{
		if ((x >= mHiliteAreaDest.x1) && (x <= mHiliteAreaDest.x2)
      		    && (y >= mHiliteAreaDest.y1) && (y <= mHiliteAreaDest.y2))
		{
			return;
		}

		if ((x >= mHiliteArea.x1) && (x <= mHiliteArea.x2)
      		    && (y >= mHiliteArea.y1) && (y <= mHiliteArea.y2))
		{
			x -= (mHiliteArea.x1 - mHiliteAreaDest.x1);
			y -= (mHiliteArea.y1 - mHiliteAreaDest.y1);
		}
	}

	int index;

	if (mUsealt == true)
	{
		if (x >= mCurwidth)
			return;
		if (y >= mCurheight)
			return;
	}

	int line = mCurwidth / 4;
	if ((mCurwidth % 4) > 0)
		line++;

	index = (y*line) + (x/4);

	color = color & 0x03;
	if ((x % 4) == 3)
	{
		mOsdBuffer[40+index] = (mOsdBuffer[40+index] & 0xFC) | color;
	}
	else if ((x % 4) == 2)
	{
		mOsdBuffer[40+index] = (mOsdBuffer[40+index] & 0xF3) | (color << 2);
	}
	else if ((x % 4) == 1)
	{
		mOsdBuffer[40+index] = (mOsdBuffer[40+index] & 0xCF) | (color << 4);
	}
	else //if ((x % 4) == 0)
	{
		mOsdBuffer[40+index] = (mOsdBuffer[40+index] & 0x3F) | (color << 6);
	}
#else
	if (x >= mCurwidth)//EM84OSD_ALTER_WIDTH)
		return;
	if (y >= mCurheight)//EM84OSD_ALTER_HEIGHT)
		return;

	mOsdBuffer[1032+(y*mCurwidth+x)] = color;
#endif
}

void cOsdEm84xx::setOSDPaletteEntry(int palidx, unsigned char r,unsigned char g,unsigned char b,unsigned char a)
{
	unsigned char* ptr = &mOsdBuffer[8+(palidx*4)];
	unsigned char temp[3];
	double R,G,B,y,u,v;
		
	R=r;
	G=g;
	B=b;
	rgbtoyuv(R,G,B,&y,&u,&v);
	temp[0] = (unsigned char) y;
	temp[1] = (unsigned char) u;
	temp[2] = (unsigned char) v;

	*ptr++ = a;
	*ptr++ = temp[0];
	*ptr++ = temp[1];
	*ptr++ = temp[2];
}

void cOsdEm84xx::rgbtoyuv(double R,double G,double B,double *y,double *u,double *v)
{
	*y=(0.257*R + 0.504*G + 0.098*B) + 16.0;
	*u=(-0.148*R - 0.291*G + 0.439*B) + 128.0;
	*v=(0.439*R - 0.368*G - 0.071*B) + 128.0;
}

cOsdProviderEm84xx::cOsdProviderEm84xx(void)
{
	Dprintf("%s\n", __PRETTY_FUNCTION__);
}

cOsd *cOsdProviderEm84xx::CreateOsd(int Left, int Top)
{
	Dprintf("%s(%d,%d)\n", __PRETTY_FUNCTION__, Left, Top);
	return new cOsdEm84xx(Left, Top);
}
